@@ -94,16 +94,16 @@ class ApiSpaces(ApiBase):
     def repository_relationship(self, space_name: str, deployment_type: str):
         return self.build_route(f"spaces/{space_name}/repository_relationships/{deployment_type}")
 
-    def custom_space_tags(self, space_name: str):
+    def space_tags(self, space_name: str):
         return self.build_route(f"spaces/{space_name}/settings/tags")
 
-    def custom_space_tag(self, space_name: str, name: str):
+    def space_tag(self, space_name: str, name: str):
         return self.build_route(f"spaces/{space_name}/settings/tags/{name}")
 
-    def custom_blueprint_tags(self, space_name: str, blueprint_name: str, deployment_type: str):
+    def blueprint_tags(self, space_name: str, blueprint_name: str, deployment_type: str):
         return self.build_route(f"spaces/{space_name}/blueprints/{blueprint_name}/settings/tags?type={deployment_type}")
 
-    def custom_blueprint_tag(self, space_name: str, blueprint_name: str, name: str, deployment_type: str):
+    def blueprint_tag(self, space_name: str, blueprint_name: str, name: str, deployment_type: str):
         return self.build_route(f"spaces/{space_name}/blueprints/{blueprint_name}/settings/tags/{name}?type={deployment_type}")
 
     def available_space_terraform_modules(self, space_name: str) -> str:
