@@ -9,6 +9,7 @@ import isodate
 from isodate import duration_isoformat
 from requests import Response
 
+from common.test_utils import TestUtils
 from gateways.apis.api_route_base import ApiRoot
 from gateways.common.cs18_api_classes import BlueprintRepositoryDetails, AccessLink, \
     AddBlueprintUsingTokenRepositoryRequest, AddBlueprintGithubRepositoryRequest, \
@@ -21,7 +22,7 @@ from gateways.common.cs18_api_requests import (
     UpdateSpaceRequest,
     ProductionEnvironment, DebuggingServiceValue, SpaceCloudAccountInfraSettingsRequest,
     AddK8SComputeServiceToSpaceRequest, UpdateK8SComputeServiceInSpaceRequest, AwsCloudProviderSettingsModel,
-    NewUserSurveyRequest, AddSpaceTerraformModuleRequest, UpdateSpaceTerraformModuleRequest)
+    NewUserSurveyRequest, AddSpaceTerraformModuleRequest, UpdateSpaceTerraformModuleRequest, AddSpaceHelmChartRequest)
 from gateways.common.cs18_api_responses import (AccountStatusResponse, BlueprintFileResponse, BlueprintResponse,
                                                 CatalogForGetAllResponse, CatalogForGetResponse, CloudAccountResponse,
                                                 CreateProductionResponse, CreateSandboxResponse, GetSpaceResponse,
@@ -35,8 +36,8 @@ from gateways.common.cs18_api_responses import (AccountStatusResponse, Blueprint
                                                 DirectAwsSshResponse, DirectAzureSshResponse, DirectRdpResponse,
                                                 ParameterStoreItem, RepositoryProviderResponse,
                                                 RepositoryRelationshipResponse, AccountExtraDetails,
-                                                TerraformModuleDescriptorResponse, CustomTagOverrideResponse,
-                                                SpaceTerraformModuleResponse, CustomTagWithStatisticsResponse)
+                                                TerraformModuleDescriptorResponse, TagOverrideResponse,
+                                                SpaceTerraformModuleResponse, TagWithStatisticsResponse, TagType)
 from gateways.n_session import NSession
 from gateways.the_gateway import TheGateway
 from gateways.utils import GatewayUtils, Utils
@@ -352,6 +353,16 @@ class Colony:
         resp = self.session.put(url=method_url)
         GatewayUtils.handle_response(resp, return_codes=[200])
 
+    def remove_cloud_account(self, cloud_account_name: str):
+        method_url = self._api_root.settings.cloud_accounts()
+
+        request = {
+            'cloud_account_name': cloud_account_name
+        }
+
+        resp = self.session.delete(url=method_url, json=request)
+        GatewayUtils.handle_response(response=resp, return_codes=[200])
+
     def get_account_template_url(self, external_id: str):
         """ Register AWS
         :param external_id: External ID required for assume role to the customer cloud account"""
@@ -893,6 +904,11 @@ class Colony:
         GatewayUtils.handle_response(response=resp, return_codes=[200])
         return resp.content
 
+    def get_api_response(self, method_url: str) -> bytes:
+        resp = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=resp, return_codes=[200])
+        return resp.content
+
     def validate_blueprint(self, blueprint: str, deployment_type: str = 'sandbox', branch: str = None,
                            commit: str = None) -> BlueprintValidationResponse:
         method_url = self._api_root.blueprint_validation.blueprint_validation_by_space_name(space_name=self.space)
@@ -1414,15 +1430,23 @@ class Colony:
     ):
         method_url = self._api_root.settings.repositories()
 
+        connection_details = None
+        if repository_details.repository_type == TestUtils.GITHUB_REPOSITORY_TYPE:
+            connection_details = {
+                "token": repository_details.access_token
+            }
+        elif repository_details.repository_type == TestUtils.BITBUCKET_REPOSITORY_TYPE:
+            connection_details = {
+                "refresh_token": repository_details.access_token
+            }
+
         request = {
             "provider_id": repository_provider_id,
             "name": repository_name,
             "allow_sharing": True,
             "open_access": True,
             "repository_url": repository_details.repository_url,
-            "connection_details": {
-                "token": repository_details.access_token
-            },
+            "connection_details": connection_details,
         }
 
         if repo_relationship:
@@ -1530,13 +1554,25 @@ class Colony:
         GatewayUtils.handle_response(response=resp, return_codes=[200])
         return resp.text.replace('"', '')
 
-    def get_terraform_modules_descriptors(self) -> List[TerraformModuleDescriptorResponse]:
+    def get_terraform_modules_descriptors(self, ignore_sample: bool = False) -> List[TerraformModuleDescriptorResponse]:
         method_url = self._api_root.settings.terraform_modules()
         resp = self.session.get(url=method_url)
         GatewayUtils.handle_response(response=resp, return_codes=[200])
         json_result = json.loads(resp.text)
-        return [Converters.create_terraform_module_descriptor_response(terraform_module)
-                for terraform_module in json_result]
+        tf_modules = [Converters.create_terraform_module_descriptor_response(terraform_module)
+                      for terraform_module in json_result]
+        if ignore_sample:
+            tf_modules = [module for module in tf_modules if "-terraform-module--Sample" not in module.name]
+        return tf_modules
+
+    def get_terraform_modules_descriptor(self, module_name: str) -> TerraformModuleDescriptorResponse:
+        method_url = self._api_root.settings.terraform_modules()
+        resp = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=resp, return_codes=[200])
+        json_result = json.loads(resp.text)
+        tf_modules = [Converters.create_terraform_module_descriptor_response(terraform_module)
+                      for terraform_module in json_result]
+        return next((module for module in tf_modules if module.name == module_name), None)
 
     def delete_terraform_modules_descriptor(
             self,
@@ -1590,19 +1626,19 @@ class Colony:
         GatewayUtils.handle_response(response=res, return_codes=[200])
         return json.loads(res.text)
 
-    def get_custom_tags(self, filter_pre_defined: bool = True) -> List[CustomTagWithStatisticsResponse]:
-        method_url = self._api_root.settings.custom_tags()
+    def get_tags(self, filter_pre_defined: bool = True) -> List[TagWithStatisticsResponse]:
+        method_url = self._api_root.settings.tags()
         res = self.session.get(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
-        custom_tags_json = json.loads(res.text)
-        custom_tag_list = [Converters.create_custom_tag_with_statistics_response(custom_tag) for custom_tag in
-                           custom_tags_json]
+        tags_json = json.loads(res.text)
+        tag_list = [Converters.create_tag_with_statistics_response(tag) for tag in
+                           tags_json]
         if filter_pre_defined:
-            custom_tag_list = list(filter(lambda x: (x.tag_type != 'pre_defined'), custom_tag_list))
-        return custom_tag_list
+            tag_list = list(filter(lambda x: (x.tag_type != TagType.PRE_DEFINED), tag_list))
+        return tag_list
 
-    def create_custom_tag(self, name: str, value: str, scope: str, possible_values: [], description: str):
-        method_url = self._api_root.settings.custom_tags()
+    def create_tag(self, name: str, value: str, scope: str, possible_values: [], description: str):
+        method_url = self._api_root.settings.tags()
         request = {
             "name": name,
             "value": value,
@@ -1613,9 +1649,9 @@ class Colony:
         res = self.session.post(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def update_custom_tag(self, old_name: str, name: str, value: str, scope: str, possible_values: [],
-                          description: str, force_remove_possible_values: bool = False):
-        method_url = self._api_root.settings.custom_tag(old_name)
+    def update_tag(self, old_name: str, name: str, value: str, scope: str, possible_values: [],
+                   description: str, force_remove_possible_values: bool = False):
+        method_url = self._api_root.settings.tag(old_name)
         request = {
             "name": name,
             "value": value,
@@ -1627,20 +1663,20 @@ class Colony:
         res = self.session.put(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def delete_custom_tag(self, name: str):
-        method_url = self._api_root.settings.custom_tag(name)
+    def delete_tag(self, name: str):
+        method_url = self._api_root.settings.tag(name)
         res = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def get_space_custom_tags(self, space_name: str) -> List[CustomTagOverrideResponse]:
-        method_url = self._api_root.spaces.custom_space_tags(space_name)
+    def get_space_tags(self, space_name: str) -> List[TagOverrideResponse]:
+        method_url = self._api_root.spaces.space_tags(space_name)
         res = self.session.get(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
-        custom_tags_json = json.loads(res.text)
-        return [Converters.create_custom_tag_override_response(custom_tag) for custom_tag in custom_tags_json]
+        tags_json = json.loads(res.text)
+        return [Converters.create_tag_override_response(tag) for tag in tags_json]
 
-    def create_space_custom_tag(self, space_name: str, name: str, value: str):
-        method_url = self._api_root.spaces.custom_space_tags(space_name)
+    def create_space_tag(self, space_name: str, name: str, value: str):
+        method_url = self._api_root.spaces.space_tags(space_name)
         request = {
             "name": name,
             "value": value
@@ -1648,30 +1684,30 @@ class Colony:
         res = self.session.post(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def update_space_custom_tag(self, space_name: str, name: str, value: str):
-        method_url = self._api_root.spaces.custom_space_tag(space_name, name)
+    def update_space_tag(self, space_name: str, name: str, value: str):
+        method_url = self._api_root.spaces.space_tag(space_name, name)
         request = {
             "value": value
         }
         res = self.session.put(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def delete_space_custom_tag(self, space_name: str, name: str):
-        method_url = self._api_root.spaces.custom_space_tag(space_name, name)
+    def delete_space_tag(self, space_name: str, name: str):
+        method_url = self._api_root.spaces.space_tag(space_name, name)
         res = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def get_blueprint_custom_tags(self, space_name: str,
-                                  blueprint_name: str, deployment_type: str) -> List[CustomTagOverrideResponse]:
-        method_url = self._api_root.spaces.custom_blueprint_tags(space_name, blueprint_name, deployment_type)
+    def get_blueprint_tags(self, space_name: str,
+                           blueprint_name: str, deployment_type: str) -> List[TagOverrideResponse]:
+        method_url = self._api_root.spaces.blueprint_tags(space_name, blueprint_name, deployment_type)
         res = self.session.get(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
-        custom_tags_json = json.loads(res.text)
-        return [Converters.create_custom_tag_override_response(custom_tag) for custom_tag in custom_tags_json]
+        tags_json = json.loads(res.text)
+        return [Converters.create_tag_override_response(tag) for tag in tags_json]
 
-    def create_blueprint_custom_tag(self, space_name: str,
-                                    blueprint_name: str, deployment_type: str, name: str, value: str):
-        method_url = self._api_root.spaces.custom_blueprint_tags(space_name, blueprint_name, deployment_type)
+    def create_blueprint_tag(self, space_name: str,
+                             blueprint_name: str, deployment_type: str, name: str, value: str):
+        method_url = self._api_root.spaces.blueprint_tags(space_name, blueprint_name, deployment_type)
         request = {
             "name": name,
             "value": value
@@ -1679,17 +1715,17 @@ class Colony:
         res = self.session.post(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def update_blueprint_custom_tag(self, space_name: str,
-                                    blueprint_name: str, deployment_type: str, name: str, value: str):
-        method_url = self._api_root.spaces.custom_blueprint_tag(space_name, blueprint_name, name, deployment_type)
+    def update_blueprint_tag(self, space_name: str,
+                             blueprint_name: str, deployment_type: str, name: str, value: str):
+        method_url = self._api_root.spaces.blueprint_tag(space_name, blueprint_name, name, deployment_type)
         request = {
             "value": value
         }
         res = self.session.put(url=method_url, json=request)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
-    def delete_blueprint_custom_tag(self, space_name: str, blueprint_name: str, deployment_type: str, name: str):
-        method_url = self._api_root.spaces.custom_blueprint_tag(space_name, blueprint_name, name, deployment_type)
+    def delete_blueprint_tag(self, space_name: str, blueprint_name: str, deployment_type: str, name: str):
+        method_url = self._api_root.spaces.blueprint_tag(space_name, blueprint_name, name, deployment_type)
         res = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
@@ -1714,6 +1750,37 @@ class Colony:
         res = self.session.delete(url=method_url)
         GatewayUtils.handle_response(response=res, return_codes=[200])
 
+    def get_space_helm_charts(self):
+        method_url = self._api_root.spaces.space_helm_charts(space_name=self.space)
+        response = self.session.get(url=method_url)
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+        json_result = json.loads(response.text)
+        return [Converters.create_space_helm_chart_response(item)
+                for item in json_result]
+
+    def create_space_helm_chart(self, request: AddSpaceHelmChartRequest):
+        method_url = self._api_root.spaces.space_helm_charts(space_name=self.space)
+        response = self.session.post(url=method_url, json=Utils.get_as_json(request))
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
+    def update_space_helm_chart(self, chart_name: str, request: AddSpaceHelmChartRequest):
+        method_url = self._api_root.spaces.space_helm_chart(space_name=self.space, helm_chart=chart_name)
+        response = self.session.put(url=method_url, json=Utils.get_as_json(request))
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
+    def delete_space_helm_chart(self, chart_name: str):
+        method_url = self._api_root.spaces.space_helm_chart(space_name=self.space, helm_chart=chart_name)
+        response = self.session.delete(url=method_url)
+        GatewayUtils.handle_response(response=response, return_codes=[200])
+
+    def forgot_account(self, email: str):
+        forgot_account_url = self._api_root.account.forgot_account()
+        request = {
+            "email": email
+        }
+        response = self.session.post(url=forgot_account_url, json=request)
+        GatewayUtils.handle_response(response, [200])
+
 
 class SandboxesHandler:
     def __init__(self):
