@@ -1238,7 +1238,8 @@ class RepositoryRelationshipResponse:
 
 
 class SpaceAdmin:
-    def __init__(self, first_name: str, last_name: str, display_first_name: str, display_last_name: str, email: str, account_role: str):
+    def __init__(self, first_name: str, last_name: str, display_first_name: str, display_last_name: str, email: str,
+                 account_role: str):
         self.last_name = last_name
         self.first_name = first_name
         self.display_last_name = display_last_name
